import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import com.mindfusion.drawing.*;
import com.mindfusion.diagramming.*;


public class FlowCharterFrame extends JFrame
{
	public FlowCharterFrame()
	{
		initDiagram();
		initLists();
		initFrame();
    }

	private void initDiagram()
	{
        hiddenDiagram = new Diagram();
        hiddenDiagram.setLinkBaseShapeSize(5);
        hiddenDiagram.setLinkHeadShapeSize(5);
        hiddenDiagram.setLinkIntermediateShapeSize(5);
		hiddenDiagram.setDefaultShape(com.mindfusion.diagramming.Shape.fromId("Decision"));
		hiddenDiagram.getRoutingOptions().setGridSize(16);
        hiddenDiagram.getRoutingOptions().setNodeVicinitySize(48);
        hiddenDiagram.setSelectAfterCreate(false);
        hiddenDiagram.setBounds(new Rectangle2D.Float(0, 0, 30, 30));
        hiddenDiagram.getShapeNodeStyle().setBrush(new SolidBrush(new Color(0,191,255)));
        hiddenDiagram.setBackBrush(new SolidBrush(Color.white));
		hiddenDiagram.setShadowOffset(0,0);
		hiddenDiagram.getDiagramLinkStyle().setBrush(new SolidBrush(new Color(51, 255, 51)));
		hiddenDiagram.setBounds(new Rectangle2D.Float(0, 0, 10, 10));
		
		DiagramView hiddenDiagramView = new DiagramView();
		hiddenDiagramView.setInplaceEditFont(new Font("Verdana", Font.BOLD, 4));
		hiddenDiagramView.setVisible(false);
		hiddenDiagramView.setDiagram(hiddenDiagram);
		this.getContentPane().add(hiddenDiagramView);

		diagram = new Diagram();		
        diagram.setLinkBaseShapeSize(3);
		diagram.getDiagramLinkStyle().setBrush(new SolidBrush(new Color( 51, 255, 51)));
		diagram.setLinkHeadShapeSize(6);
		diagram.setLinkIntermediateShapeSize(3);
		diagram.setBackBrush(new SolidBrush(new Color((int)202, (int)202, (int)202)));
		diagram.setSelectAfterCreate(false);
		
		ShapeNodeStyle style = diagram.getShapeNodeStyle();
		style.setFontFamily("Arial");
		style.setFontSize(4);
		style.setFontUnit(GraphicsUnit.Millimeter);
		style.setFontStyle(0);
		
		diagram.getUndoManager().setUndoEnabled(true);
		diagram.setGridColor(Color.black);
		diagram.setShowAnchors(ShowAnchors.Auto);
		diagram.setShowGrid(true);
		diagram.setAlignToGrid(true);
        diagram.setSnapToAnchor(SnapToAnchor.OnCreateOrModify);
        diagram.getShapeNodeStyle().setBrush(new SolidBrush(new Color(0,191,255)));
        diagram.setMeasureUnit(GraphicsUnit.Millimeter);
        diagram.setBounds(new Rectangle2D.Float(0,0,380,380));
        diagram.setAlignToGrid(true);

        diagram.addDiagramListener(new DiagramAdapter()
        {
        	public void nodeClicked(NodeEvent e)
        	{
        		if(e.getNode() instanceof ShapeNode)
        		{
        			onNodeClicked(e);
        		}
        	}

        	public void linkClicked(LinkEvent e)
        	{
        		onLinkClicked(e);
    		}

        	public void nodeSelected(NodeEvent e)
        	{
        		if(e.getNode() instanceof ShapeNode)
        		{
        			onNodeSelected();
        		}
        	}

			public void linkSelected(LinkEvent e)
			{
				onLinkSelected();
			}

			public void nodeDeselected(NodeEvent e)
        	{
        		onDeselected();
        	}

			public void linkDeselected(LinkEvent e)
			{
				onDeselected();
			}
        });

        diagramView = new DiagramView();
        diagramView.setDiagram(diagram);
        diagramView.setCursor(Cursor.getDefaultCursor());
		diagramView.setBehavior(Behavior.DrawLinks);
        diagramView.setTransferHandler(_transferHandler = new FlowChartTransferHandler());
        diagramView.addMouseMotionListener(new MouseMotionAdapter()
        {
        	public void mouseMoved(MouseEvent e)
        	{
        		if(FlowChartTransferHandler.isNodeCreated())
        		{
        			onNodeCreating(e);
        		}
        	}
        });
	}

	private void initLists()
	{
		_nodes = new NodeInfo[]
		{
			 new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red)
				}, "Alternative"),
				com.mindfusion.diagramming.Shape.fromId("Alternative"),	// get shape by string id
				"Alternative"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 30,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  30, true, true, MarkStyle.Circle, Color.red)
				}, "Card"),
				com.mindfusion.diagramming.Shape.fromId("PunchedCard"),
				"Card"),
			new NodeInfo(
				AnchorPattern.fromId("Decision2In2Out"),
				com.mindfusion.diagramming.Shape.fromId("Decision"),
				"Decision"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Delay"),
				com.mindfusion.diagramming.Shape.fromId("Delay"),
				"Delay"),
			new NodeInfo(
			new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  20, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  80, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  80, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  20, true, true, MarkStyle.Circle, Color.red)
				}, "DiskStorage"),
				com.mindfusion.diagramming.Shape.fromId("DiskStorage"),
				"Disk Storage"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 40,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 85,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 85,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 40,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Display"),
				com.mindfusion.diagramming.Shape.fromId("Display"),
				"Display"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Divided event"),
				com.mindfusion.diagramming.Shape.fromId("DividedEvent"),
				"Divided event"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Divided process"),
				com.mindfusion.diagramming.Shape.fromId("DividedProcess"),
				"Divided process"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Document"),
				com.mindfusion.diagramming.Shape.fromId("Document"),
				"Document"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Internal storage"),
				com.mindfusion.diagramming.Shape.fromId("InternalStorage"),
				"Internal storage"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 25,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 75,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  25, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  25, true, true, MarkStyle.Circle, Color.red)
				}, "Loop limit"),
				com.mindfusion.diagramming.Shape.fromId("BeginLoop"),
				"Loop limit"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 50,(short)  20, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 40, true, true, MarkStyle.Circle, Color.red)
				}, "Manual input"),
				com.mindfusion.diagramming.Shape.fromId("Input"),
				"Manual input"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 80,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 20,(short) 100, true, true, MarkStyle.Circle, Color.red)
				}, "Manual operation"),
				com.mindfusion.diagramming.Shape.fromId("ManualOperation"),
				"Manual operation"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 80,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Message from User"),
				com.mindfusion.diagramming.Shape.fromId("MessageFromUser"),
				"Message from User"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 20,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 20,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Message to User"),
				com.mindfusion.diagramming.Shape.fromId("MessageToUser"),
				"Message to User"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 45,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Multi-document"),
				com.mindfusion.diagramming.Shape.fromId("MultiDocument"),
				"Multi-document"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)  10, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)  10, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  10, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  90, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Paper tape"),
				com.mindfusion.diagramming.Shape.fromId("Microform"),
				"Paper tape"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 80,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Primitive from Call"),
				com.mindfusion.diagramming.Shape.fromId("PrimitiveFromCall"),
				"Primitive from Call"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 20,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 20,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Primitive to Call"),
				com.mindfusion.diagramming.Shape.fromId("PrimitiveToCall"),
				"Primitive to Call"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short)  0,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red)
				}, "Procedure"),
				com.mindfusion.diagramming.Shape.fromId("Procedure"),
				"Procedure"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 15,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 85,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  30, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  70, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 15,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 85,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  30, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  70, true, true, MarkStyle.Circle, Color.red)
				}, "Start"),
				com.mindfusion.diagramming.Shape.fromId("Start"),
				"Start"),
			new NodeInfo(
				new AnchorPattern(new AnchorPoint[]
				{
					new AnchorPoint((short) 50,(short)   0, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)  0,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short) 50,(short) 100, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short)  50, true, true, MarkStyle.Circle, Color.red),
					new AnchorPoint((short)100,(short) 100, true, true, MarkStyle.Circle, Color.red)
				}, "Tape"),
				com.mindfusion.diagramming.Shape.fromId("Tape"),
				"Tape")
		};

        _nodeList = new JList<String>();
        _nodeList.setCellRenderer(nodeRenderer);
        _nodeList.setDoubleBuffered(true);
        _nodeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        _scrollNodeList = new JScrollPane(_nodeList);
        _scrollNodeList.setAutoscrolls(true);

        _connectors = new ConnectorInfo[]
		{
			new ConnectorInfo(
				ArrowHeads.Arrow,
				"Arrow"),
			new ConnectorInfo(
				ArrowHeads.BackSlash,
				"Back slash"),
			new ConnectorInfo(
				ArrowHeads.BowArrow,
				"Bow arrow"),
			new ConnectorInfo(
				ArrowHeads.Circle,
				"Circle"),
			new ConnectorInfo(
				ArrowHeads.DoubleArrow,
				"Double arrow"),
			new ConnectorInfo(
				ArrowHeads.None,
				"None"),
			new ConnectorInfo(
				ArrowHeads.Pentagon,
				"Pentagon"),
			new ConnectorInfo(
				ArrowHeads.PointerArrow,
				"Pointer"),
			new ConnectorInfo(
				ArrowHeads.Quill,
				"Quill"),
			new ConnectorInfo(
				ArrowHeads.Reversed,
				"Reversed"),
			new ConnectorInfo(
				ArrowHeads.RevTriangle,
				"Reversed triangle"),
			new ConnectorInfo(
				ArrowHeads.RevWithCirc,
				"Reversed with Circle"),
			new ConnectorInfo(
				ArrowHeads.RevWithLine,
				"Reversed with Line"),
			new ConnectorInfo(
				ArrowHeads.Rhombus,
				"Rhombus"),
			new ConnectorInfo(
				ArrowHeads.Slash,
				"Slash"),
			new ConnectorInfo(
				ArrowHeads.Tetragon,
				"Tetragon"),
			new ConnectorInfo(
				ArrowHeads.Triangle,
				"Triangle")
		};

        _connectorList = new JList<String>();
		_connectorList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		_scrollConnectorsList = new JScrollPane(_connectorList);
		_scrollConnectorsList.setAutoscrolls(true);
	}

	private void initFrame()
	{
		//File
        _menuBar = new JMenuBar();
        _menuFile = new JMenu();
		_menuItemNew = new JMenuItem();
		_menuItemOpen = new JMenuItem();
		_menuItemSave = new JMenuItem();
		_menuItemPrint = new JMenuItem();
		_menuItemPrintPreview = new JMenuItem();
		_menuItemExit = new JMenuItem();

		//Edit
		_menuEdit = new JMenu();
		_menuItemUndo = new JMenuItem();
		_menuItemRedo = new JMenuItem();

		//Tools and help
		_menuTools = new JMenu();
		_menuItemOptions = new JMenuItem();
		_menuHelp = new JMenu();
		_menuItemAbout = new JMenuItem();

		_cp = this.getContentPane();
    	_scrollPane = new JScrollPane(diagramView);
    	_rightPanel = new JPanel();
		_leftPanel = new JPanel();
		_tabRightPanel = new JPanel();
		_tabPane = new JTabbedPane ();
		_linkPropertyGrid = new JPanel();
		_nodePropertyGrid = new JPanel();

    	_connectorTypeCombo = new JComboBox<String>(_typeArrow);
    	_fileChooser = new JFileChooser ();
		_popupMenu = new JPopupMenu ();
		_menuItemDelete = new JMenuItem ();

	    //
        //_menuBar
        //
        _menuBar.setName("_menuBar");
        _menuBar.add(_menuFile);
        _menuBar.add(_menuEdit);
        _menuBar.add(_menuTools);
        _menuBar.add(_menuHelp);

        //
		// _menuFile
		//
        _menuFile.setName("_menuFile");
        _menuFile.setText("File");
        _menuFile.setMnemonic(KeyEvent.VK_F);
        _menuFile.add(_menuItemNew);
        _menuFile.add(_menuItemOpen);
        _menuFile.add(_menuItemSave);
        _menuFile.addSeparator();
        _menuFile.add(_menuItemPrint);
        _menuFile.add(_menuItemPrintPreview);
        _menuFile.addSeparator();
        _menuFile.add(_menuItemExit);


        //
        //_menuItemNew
        //
        _menuItemNew.setName("_menuItemNew");
        _menuItemNew.setText("New");
        _menuItemNew.setMnemonic(KeyEvent.VK_N);
        _menuItemNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        _menuItemNew.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onFileNew();
        	}
        });

		//
		//_menuItemOpen
		//
        _menuItemOpen.setName("_menuItemOpen");
        _menuItemOpen.setText("Open");
        _menuItemOpen.setMnemonic(KeyEvent.VK_O);
        _menuItemOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        _menuItemOpen.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onFileOpen();
        	}
        });

        //
        //_menuItemSave
        //
        _menuItemSave.setName("_menuItemSave");
        _menuItemSave.setText("Save");
        _menuItemSave.setMnemonic(KeyEvent.VK_S);
        _menuItemSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        _menuItemSave.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		ofFileSave();
        	}
        });

        //
        //_menuItemPrint
        //
        _menuItemPrint.setName("_menuItemPrint");
        _menuItemPrint.setText("Print");
        _menuItemPrint.setMnemonic(KeyEvent.VK_P);
        _menuItemPrint.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
        _menuItemPrint.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onFilePrint();
        	}
        });

        //
        //_menuItemPrintPreview
        //
        _menuItemPrintPreview.setName("_menuItemPrintPreview");
        _menuItemPrintPreview.setText("Print Preview");
        _menuItemPrintPreview.setMnemonic(KeyEvent.VK_T);
        _menuItemPrintPreview.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
        _menuItemPrintPreview.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		diagramView.printPreview();
        	}
        });

        //
        //_menuItemExit
        //
        _menuItemExit.setName("_menuItemExit");
        _menuItemExit.setText("Exit");
        _menuItemExit.setMnemonic(KeyEvent.VK_X);
        _menuItemExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		_menuItemExit.addActionListener
        (
            new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    onFileExit();
                }
            }
        );

        //
        //_menuEdit
        //
        _menuEdit.setName("_menuEdit");
        _menuEdit.setText("Edit");
        _menuEdit.setMnemonic(KeyEvent.VK_E);
       	_menuEdit.add(_menuItemUndo);
       	_menuEdit.add(_menuItemRedo);

       	//
       	//_menuItemUndo
       	//
        _menuItemUndo.setName("_menuItemUndo");
        _menuItemUndo.setText("Undo");
        _menuItemUndo.setEnabled(_editUndoRedo);
        _menuItemUndo.setMnemonic(KeyEvent.VK_Z);
        _menuItemUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
        _menuItemUndo.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onEditUndo();
        	}
        });



        //
       	//_menuItemRedo
       	//
        _menuItemRedo.setName("_menuItemRedo");
        _menuItemRedo.setText("Redo");
        _menuItemRedo.setEnabled(_editUndoRedo);
        _menuItemRedo.setMnemonic(KeyEvent.VK_Y);
        _menuItemRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
        _menuItemRedo.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onEditRedo();
        	}
        });
 //       _menuItemRedo.setEnabled(_editUndoRedo);

        //
        //_menuTools
        //
        _menuTools.setName("_menuTools");
        _menuTools.setText("Tools");
        _menuTools.setMnemonic(KeyEvent.VK_T);
        _menuTools.add(_menuItemOptions);

        //
        //_menuItemOptions
        //
        _menuItemOptions.setName("_menuItemOptions");
        _menuItemOptions.setText("Options");
        _menuItemOptions.setMnemonic(KeyEvent.VK_O);
        _menuItemOptions.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        _menuItemOptions.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onToolsOptions();
        	}
        });

        //
        //_menuHelp
        //
        _menuHelp.setName("_menuHelp");
        _menuHelp.setText("Help");
        _menuHelp.setMnemonic(KeyEvent.VK_H);
        _menuHelp.add(_menuItemAbout);

        //
        //_menuItemAbout
        //
        _menuItemAbout.setName("_menuItemAbout");
        _menuItemAbout.setText("About ... ");
        _menuItemAbout.setMnemonic(KeyEvent.VK_A);
        _menuItemAbout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        _menuItemAbout.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onHelpAbout();
        	}
        });

        //
        //_menuItemDelete
        //
        _menuItemDelete.setName("_menuItemDelete");
        _menuItemDelete.setText("Delete");
        _menuItemDelete.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		onDelete();
        	}
        });

        //
        //_popupMenu
        //
        _popupMenu.setName("_popupMenu");
        _popupMenu.add(_menuItemDelete);

        //
		// _nodeList
		//
		_nodeList.setName("_nodeList");
		_nodeList.setBorder(new LineBorder(new Color((int)25, (int)25, (int)112)));
        _nodeList.setDragEnabled(true);
        _nodeList.setTransferHandler(_transferHandler);
        _nodeList.addMouseListener(new MouseAdapter()
        {
        	public void mousePressed(MouseEvent e)
        	{

        	}
        });
        _nodeList.addListSelectionListener(new ListSelectionListener(){
        	public void	valueChanged(ListSelectionEvent e)
        	{
        		NodeList_OnMouseDown(e);
			}
        });

        //
        //_connectorList
        //
        _connectorList.setName("_connectorList");
        _connectorList.setBorder(new LineBorder(new Color((int)25, (int)25, (int)112)));
        _connectorList.setDragEnabled(false);
        _connectorList.addMouseListener(new MouseAdapter(){

        });
        _connectorList.addListSelectionListener(new ListSelectionListener()
        {
        	public void valueChanged(ListSelectionEvent e)
        	{
        		if(e.getValueIsAdjusting() == true)
        			ConnectorList_OnSelectionChanged();
        	}
        });


        //
        //_connectorTypeCombo
        //
        _connectorTypeCombo.setName("_connectorTypeCombo");
        _connectorTypeCombo.setBackground(Color.white);
        _connectorTypeCombo.addActionListener(new ActionListener()
        {
        	public void actionPerformed(ActionEvent e)
        	{
        		ConnectorTypeCombo_OnSelectedValueChange();
        	}
        });
        
        //
        //_scrollPane
        //
        _scrollPane.setName("_scrollPane");
		_scrollPane.setAutoscrolls(true);
		
        //
        //_leftPanel
        //
        _leftPanel.setLayout(new BoxLayout(_leftPanel, BoxLayout.Y_AXIS));
        _leftPanel.add(_scrollPane);

        //
		//_tabRightPanel
		//
		_tabRightPanel.setLayout(null);
        _tabRightPanel.setName("_tabRightPanel");
       	_tabRightPanel.add(_scrollConnectorsList);
       	_tabRightPanel.add(_connectorTypeCombo);

        createNodePropertyGrid();
        createLinkPropertyGrid();

        javax.swing.SpringLayout _panelSpringLayout = new javax.swing.SpringLayout();

        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.NORTH, _scrollConnectorsList, 0,
        									javax.swing.SpringLayout.NORTH, _tabRightPanel);
        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.WEST, _tabRightPanel, 0,
        									javax.swing.SpringLayout.WEST, _scrollConnectorsList);
        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.EAST, _tabRightPanel, 0,
        									javax.swing.SpringLayout.EAST, _scrollConnectorsList);
        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.SOUTH, _tabRightPanel, 25,
        									javax.swing.SpringLayout.SOUTH, _scrollConnectorsList);

        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.NORTH, _tabRightPanel, 0,
        									javax.swing.SpringLayout.NORTH, _connectorTypeCombo);
        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.EAST, _connectorTypeCombo, 0,
        									javax.swing.SpringLayout.EAST, _tabRightPanel);
        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.SOUTH, _connectorTypeCombo, 0,
        									javax.swing.SpringLayout.SOUTH, _tabRightPanel);
        _panelSpringLayout.putConstraint(javax.swing.SpringLayout.WEST, _connectorTypeCombo, 0,
        									javax.swing.SpringLayout.WEST, _tabRightPanel);

        _tabRightPanel.setLayout(_panelSpringLayout);

        //
        //_tabPane
        //
        _tabPane.setName("_tabPane");
		_tabPane.addTab("Nodes", _scrollNodeList);
		_tabPane.addTab("Connectors", _tabRightPanel);

        //
        //_rightPanel
        //
        _rightPanel.setName("_rightPanel");
        _rightPanel.setLayout(new BoxLayout(_rightPanel, BoxLayout.Y_AXIS));
        _rightPanel.setPreferredSize(new Dimension(220, 750));
        _rightPanel.add(_tabPane);


        //
        //_nodePropertyGrid
        //
        _nodePropertyGrid.setName("_nodePropertyGrid");
        _nodePropertyGrid.setLayout(null);
        _nodePropertyGrid.setPreferredSize(new Dimension(220,141));
        _nodePropertyGrid.setVisible(false);

        //
        //_linkPropertyGrid
        //
        _linkPropertyGrid.setName("_linkPropertyGrid");
        _linkPropertyGrid.setLayout(null);
        _linkPropertyGrid.setPreferredSize(new Dimension(220,141));
        _linkPropertyGrid.setVisible(false);

        _cp.setLayout(null);
        _cp.add(_leftPanel);
        _cp.add(_rightPanel);
        _cp.add(_nodePropertyGrid);
        _cp.add(_linkPropertyGrid);

        javax.swing.SpringLayout _sprLayout = new javax.swing.SpringLayout();
        _sprLayout.putConstraint(javax.swing.SpringLayout.EAST, _cp, 220,
        							javax.swing.SpringLayout.EAST, _leftPanel);
        _sprLayout.putConstraint(javax.swing.SpringLayout.SOUTH, _leftPanel, 0,
        							javax.swing.SpringLayout.SOUTH, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, _leftPanel, 0,
        							javax.swing.SpringLayout.NORTH, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.EAST, _rightPanel, 0,
        							javax.swing.SpringLayout.EAST, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.SOUTH, _rightPanel, -141,
        							javax.swing.SpringLayout.SOUTH, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.NORTH, _rightPanel, 0,
        							javax.swing.SpringLayout.NORTH, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.SOUTH, _nodePropertyGrid, 0,
        							javax.swing.SpringLayout.SOUTH, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.EAST, _nodePropertyGrid, 0,
        							javax.swing.SpringLayout.EAST, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.SOUTH, _linkPropertyGrid, 0,
        							javax.swing.SpringLayout.SOUTH, _cp);
        _sprLayout.putConstraint(javax.swing.SpringLayout.EAST, _linkPropertyGrid, 0,
        							javax.swing.SpringLayout.EAST, _cp);

        _cp.setLayout(_sprLayout);


        setTitle("MindFusion.Diagramming Sample: Flowcharter");
        setJMenuBar(_menuBar);
        setSize(new Dimension(600, 600));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Add window listener.
        this.addWindowListener
        (
            new WindowAdapter()
            {
                public void windowClosing(WindowEvent e)
                {
                    FlowCharterFrame.this.windowClosed();
                }
            }
        );
	}

    @Override
    public void setVisible(boolean value)
    {
    	super.setVisible(value);

		//node renderer
		DefaultListModel<String> _modelN = new DefaultListModel<String>();
		nodeRenderer = new ListRenderer(_nodes.length);

        int _position = 0;
        for (NodeInfo s : _nodes)
        {
        	_modelN.addElement(s.getText());

 	      	ShapeNode node = hiddenDiagram.getFactory().createShapeNode(1,1,8,8);
 	      	node.setShape(_nodes[_position].getShape());

       		BufferedImage image = hiddenDiagram.createImage();

       		nodeRenderer.createBufferedImage(_position, s.getText(), image);

        	hiddenDiagram.clearAll();
        	_position++;
        }

		_nodeList.setModel(_modelN);
		_nodeList.setCellRenderer(nodeRenderer);


		//connector renderer
		connectorRenderer = new ListRenderer(_nodes.length);

        _position = 0;
        DefaultListModel<String> _modelC = new DefaultListModel<String>();

        for(ConnectorInfo s : _connectors)
        {
        	_modelC.addElement(s.getName());

        	ShapeNode b1, b2;
 			DiagramLink a;


 			b1 = hiddenDiagram.getFactory().createShapeNode(-4, 1, -2, 8);
			b2 = hiddenDiagram.getFactory().createShapeNode(8, 1, 11, 8);

			a = hiddenDiagram.getFactory().createDiagramLink(b1, b2);

			b1.setVisible(false);
			b2.setVisible(false);

			a.setHeadShape(_connectors[_position].getHead());

			BufferedImage image = hiddenDiagram.createImage();

			connectorRenderer.createBufferedImage(_position, s.getName(), image);

			hiddenDiagram.clearAll();
			_position++;
        }

        _connectorList.setModel(_modelC);
        _connectorList.setCellRenderer(connectorRenderer);
    }

    /**
     * Shutdown procedure when run as an application.
     */
    protected void windowClosed()
    {
    	// TODO: Check if it is safe to close the application

        // Exit application.
        System.exit(0);
    }

    /**
	 * Handles the File.New menu event.
	 */
    public void onFileNew()
    {
    	diagram.clearAll();
    }

    /**
	 * Handles the File.Open menu event.
	 */
    public void onFileOpen()
    {
    	if (_fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				diagram.loadFrom(_fileChooser.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException exp)
			{
				JOptionPane.showMessageDialog(this, exp.getMessage());
			}
			catch (IOException exp)
			{
				JOptionPane.showMessageDialog(this, exp.getMessage());
			}
		}
    }

    /**
	 * Handles the File.Save menu event.
	 */
    public void ofFileSave()
    {

    	if (_fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
    	{
			try
			{
				diagram.saveTo(_fileChooser.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException exp)
			{
				JOptionPane.showMessageDialog(this, exp.getMessage());
			}
			catch (IOException exp)
			{
				JOptionPane.showMessageDialog(this, exp.getMessage());
			}
		}
    }

    /**
     * Handles the File.Print menu event
     */
    public void onFilePrint()
    {
    	diagramView.print();
    }

    /**
     * Handles the File.Exit menu event
     */
    public void onFileExit()
    {
    	FlowCharterFrame.this.windowClosed();
    }

    public void onEditPopup()
    {
    	if(diagram.getUndoManager().getHistory().getNextUndo() != null)
    	{
    		_menuItemUndo.setEnabled(true);
    		_menuItemUndo.setText("Undo " + diagram.getUndoManager().getHistory().getNextUndo().getTitle());
    	}
    	else
    	{
    		_menuItemUndo.setEnabled(false);
    		_menuItemUndo.setText("Undo");
    	}

    	if(diagram.getUndoManager().getHistory().getNextRedo() != null)
    	{
    		_menuItemRedo.setEnabled(true);
    		_menuItemRedo.setText("Redo " + diagram.getUndoManager().getHistory().getNextRedo().getTitle());
    	}
    	else
    	{
    		_menuItemRedo.setEnabled(false);
    		_menuItemRedo.setText("Redo");
    	}

    }

    /**
     * Handles the Edit.Undo menu event
     */
    public void onEditUndo()
    {
    	diagram.getUndoManager().undo();
    }

    /**
     * Handles the Edit.Redo menu event
     */
    public void onEditRedo()
    {
    	diagram.getUndoManager().redo();
    }

    /**
     * Handles the Tools.Options menu event
     */
    public void onToolsOptions()
    {
    	OptionsDialog dlg = new OptionsDialog(this);
    	dlg.setShowGrid(diagram.getShowGrid());
    	dlg.setBackgroundColor(((SolidBrush)diagram.getBackBrush()).getColor());
    	dlg.setNodeColor(((SolidBrush)diagram.getShapeNodeStyle().getBrush()).getColor());
   		dlg.setConnectorColor(((SolidBrush)diagram.getDiagramLinkStyle().getBrush()).getColor());
   		dlg.setAnchorColor(_anchorColor);

   		dlg.setShowAnchor(diagram.getShowAnchors() != ShowAnchors.Never);

   		switch(_anchorStyle)
		{

			case Circle:
				dlg.setAnchorStyle(0);
				break;
			case Rectangle:
				dlg.setAnchorStyle(1);
				break;
			case X:
				dlg.setAnchorStyle(2);
				break;
			default:
				dlg.setAnchorStyle(0);
				break;

		}

		dlg.setVisible(true);

		if(dlg.getAccepted() == true)
		{
			diagram.setShowGrid(dlg.getShowGrid());
			diagram.setBackBrush(new SolidBrush(dlg.getBackgroundColor()));
			diagram.getShapeNodeStyle().setBrush(new SolidBrush(dlg.getNodeColor()));
			diagram.getDiagramLinkStyle().setBrush(new SolidBrush(dlg.getConnectorColor()));
			_anchorColor = dlg.getAnchorColor();
			
			if(dlg.getShowAnchor() == true)
				diagram.setShowAnchors(ShowAnchors.Auto);
			else
				diagram.setShowAnchors(ShowAnchors.Never);

			switch(dlg.getAnchorStyle())
			{
				case 0:
					_anchorStyle = MarkStyle.Circle;
					break;
				case 1:
					_anchorStyle = MarkStyle.Rectangle;
					break;
				case 2:
					_anchorStyle = MarkStyle.X;
					break;
				default:
					_anchorStyle = MarkStyle.Circle;
					break;
			}
			
			overview.repaint();
		}
	}

    /**
     * Handles the Help.About menu event
     */
    public void onHelpAbout()
    {
    	AboutDialog aboutDialog = new AboutDialog(this);
    	aboutDialog.setVisible(true);
    }

    public void NodeList_OnMouseDown(ListSelectionEvent e)
    {
    	selectedNodeIndex =_nodeList.getSelectedIndex();
    }

    public void ConnectorList_OnSelectionChanged()
    {
    	int si = _connectorList.getSelectedIndex();

    	if(si<0 || si>=_connectorList.getModel().getSize())
    		return;


    	diagram.setLinkHeadShape(_connectors[si].getHead());
    }

	public void ConnectorTypeCombo_OnSelectedValueChange()
	{
		switch(_connectorTypeCombo.getSelectedIndex())
		{
			case 0:
				diagram.setLinkShape(LinkShape.Polyline);
				diagram.setLinkSegments((short)1);
				break;
			case 1:
				diagram.setLinkShape(LinkShape.Bezier);
				diagram.setLinkSegments((short)1);
				break;
			case 2:
				diagram.setLinkShape(LinkShape.Cascading);
				diagram.setLinkSegments((short)3);
				break;
		}
	}

	public void onNodeClicked(NodeEvent e)
	{
		if (e.getMouseButton() == MouseEvent.BUTTON1)
		{
			if (!(e.getNode() instanceof ShapeNode))
				return;

			// show attributes of clicked node in the property grid
			ShapeNode node = (ShapeNode)e.getNode();
			String label = node.getText();

			ShapeNodeStyle style = node.getStyle() != null ?
				(ShapeNodeStyle)node.getStyle() : diagram.getShapeNodeStyle();
			String fontName = style.getFontFamily();
			int fontSize = (int)style.getFontSize();
			int fontStyle = style.getFontStyle();

			boolean bold = (fontStyle & Font.BOLD) != 0;
			_rBold.setSelectedIndex(bold ? 0 : 1);

			boolean italic = (fontStyle & Font.ITALIC) != 0;
			_rItalic.setSelectedIndex(italic ? 0 : 1);

			int sizeIndex = (fontSize / 2) - 2;
			if (sizeIndex >= 0 && sizeIndex < _rSize.getItemCount())
				_rSize.setSelectedIndex(sizeIndex);

			for (int i = 0; i < _fonts.length; i++)
			{
				if (fontName.compareTo(_fonts[i]) == 0)
					_rName.setSelectedIndex(i);
			}
			_rText.setText(label);
			
			Brush brush = e.getNode().getBrush() != null ?
				e.getNode().getBrush() : e.getNode().getEffectiveBrush();
			Color color = brush instanceof SolidBrush ?
				((SolidBrush)brush).getColor() : Color.white;
			setNodeFillColor(color);
		}
		else if(e.getMouseButton() == MouseEvent.BUTTON3 )
		{
			Point mousePos = getMousePosition();

			//display context menu at right mouse click
			_popupMenu.show(this, mousePos.x, mousePos.y);
		}
	}

	public void onLinkClicked(LinkEvent e)
	{
		if (e.getMouseButton() == MouseEvent.BUTTON1)
		{
			int getCountSegments = 0;
			for (DiagramLink link : diagram.getSelection().getLinks())
			{
				Brush brush = link.getBrush() != null ?
					link.getBrush() : link.getEffectiveBrush();
				Color color = brush instanceof SolidBrush ?
					((SolidBrush)brush).getColor() : Color.white;
				_rAHeadColor.setBackground(color);
				arrowHeadSize = link.getHeadShapeSize();

				if(link.getAllowMoveEnd() == true){
					_rAEndMovable.setSelectedIndex(0);
				}
				else
					_rAEndMovable.setSelectedIndex(1);

				if(link.getAllowMoveStart() == true){
					_rAStartMovable.setSelectedIndex(0);
				}
				else
					_rAStartMovable.setSelectedIndex(1);

				getCountSegments = (int)link.getSegmentCount();
			}

			switch ((int)arrowHeadSize)
			{
				case 4:
					_rAHeadSize.setSelectedIndex(0);
					break;
				case 6:
					_rAHeadSize.setSelectedIndex(1);
					break;
				case 8:
					_rAHeadSize.setSelectedIndex(2);
					break;
				case 10:
					_rAHeadSize.setSelectedIndex(3);
					break;
				case 12:
					_rAHeadSize.setSelectedIndex(4);
					break;
				case 14:
					_rAHeadSize.setSelectedIndex(5);
					break;
				case 16:
					_rAHeadSize.setSelectedIndex(6);
					break;
				case 18:
					_rAHeadSize.setSelectedIndex(7);
					break;
			}

			switch (getCountSegments)
			{
				case 1:
					_rASegments.setSelectedIndex(0);
					break;
				case 2:
					_rASegments.setSelectedIndex(1);
					break;
				case 3:
					_rASegments.setSelectedIndex(2);
					break;
				case 4:
					_rASegments.setSelectedIndex(3);
					break;
				case 5:
					_rASegments.setSelectedIndex(4);
					break;
				case 6:
					_rASegments.setSelectedIndex(5);
					break;
				case 7:
					_rASegments.setSelectedIndex(6);
					break;
			}

		}

		else if(e.getMouseButton() == MouseEvent.BUTTON3 )
		{
			Point mousePos = getMousePosition();

			//display context menu at right mouse click
			_popupMenu.show(this, mousePos.x, mousePos.y);
		}
	}

	public void onDeselected()
	{
		_nodePropertyGrid.setVisible(false);
		_linkPropertyGrid.setVisible(false);
	}

	public void onDelete()
	{
		if (!(diagram.getSelection().getNodes().isEmpty()))
			diagram.getNodes().removeAll(diagram.getSelection().getNodes());
		if (!(diagram.getSelection().getLinks().isEmpty()))
			diagram.getLinks().removeAll(diagram.getSelection().getLinks());
	}

	protected static ImageIcon createImageIcon(String path)
	{
        java.net.URL imgURL = FlowCharterFrame.class.getResource(path);
        if (imgURL != null)
        {
            return new ImageIcon(imgURL);
        }
        else
        {
            System.err.println("Couldn't find file: " + path);
                return null;
        }
    }

	public static com.mindfusion.diagramming.Shape getNodeShape()
	{
		return _nodes[selectedNodeIndex].getShape();
	}

	public static AnchorPattern getAnchorPattern()
	{
		return _nodes[selectedNodeIndex].getAnchor();
	}

	public static MarkStyle getAnchorStyle()
	{
		return _anchorStyle;
	}

	public void setAnchorStyle(MarkStyle style)
	{
		_anchorStyle = style;
	}

	public void onNodeSelected()
	{
		_linkPropertyGrid.setVisible(false);
		_linkPropertyGrid.repaint();

		_nodePropertyGrid.setVisible(true);
		_nodePropertyGrid.repaint();
	}

	public void onLinkSelected()
	{
		_nodePropertyGrid.setVisible(false);
		_nodePropertyGrid.repaint();

		_linkPropertyGrid.setVisible(true);
		_linkPropertyGrid.repaint();
	}

	private GraphicsEnvironment _graphEnvir;
	String[] _fonts;

	public void createNodePropertyGrid()
	{
		//
		//get fonts
		//
		_graphEnvir = GraphicsEnvironment.getLocalGraphicsEnvironment();
		_fonts = _graphEnvir.getAvailableFontFamilyNames();

		DefaultComboBoxModel<String> fontsModel = new DefaultComboBoxModel<String>();

		for (String m : _fonts){
			fontsModel.addElement(m);
		}

		//
		//create size model
		//
		String[] size = {"4", "6", "8", "10", "12", "14", "16", "18"};
		DefaultComboBoxModel<String> sizeModel = new DefaultComboBoxModel<String>();

		for(String m : size){
			sizeModel.addElement(m);
		}

		//
		//BoxPanel's elements
		//
		_nodePropertyGrid.setLayout(null);

		_lFillColor = new JLabel();
		_lFont = new JLabel();
		_lName = new JLabel();
		_lSize = new JLabel();
		_lBold = new JLabel();
		_lItalic = new JLabel();
		_lText = new JLabel();

		_rFillColor = new JPanel();
	 	_rFont = new JLabel();
	 	_rName = new JComboBox<String>();
	 	_rSize = new JComboBox<String>();
	 	_rBold = new JComboBox<String>();
	 	_rItalic = new JComboBox<String>();
	 	_rText = new JTextField();

		_lFillColor.setName("_lLillColor");
		_lFillColor.setText(" Fill color");
		_lFillColor.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lFillColor.setBounds(1,1, 110, 20);
		_lFillColor.setOpaque(true);
		_lFillColor.setBackground(Color.white);

		_lFont.setName("_lFont");
		_lFont.setText(" Font");
		_lFont.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lFont.setBounds(1,21, 110, 20);
		_lFont.setOpaque(true);
		_lFont.setBackground(Color.white);

		_lName.setName("_lName");
		_lName.setText("       Name");
		_lName.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lName.setBounds(1,41, 110, 20);
		_lName.setOpaque(true);
		_lName.setBackground(Color.white);

		_lSize.setName("_lSize");
		_lSize.setText("       Size");
		_lSize.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lSize.setBounds(1,61, 110, 20);
		_lSize.setOpaque(true);
		_lSize.setBackground(Color.white);

		_lBold.setName("_lBold");
		_lBold.setText("       Bold");
		_lBold.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lBold.setBounds(1,81, 110, 20);
		_lBold.setOpaque(true);
		_lBold.setBackground(Color.white);

		_lItalic.setName("_lItalic");
		_lItalic.setText("       Italic");
		_lItalic.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lItalic.setBounds(1,101, 110, 20);
		_lItalic.setOpaque(true);
		_lItalic.setBackground(Color.white);

		_lText.setName("_lText");
		_lText.setText("Text");
		_lText.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lText.setBounds(1,121, 110, 20);
		_lText.setOpaque(true);
		_lText.setBackground(Color.white);

		_rFillColor.setName("_rFillColor");
		_rFillColor.setBorder(new LineBorder(new Color(100, 100, 100)));
		_rFillColor.setBounds(111, 1, 109, 20);
		_rFillColor.setBackground(Color.white);
		_rFillColor.addMouseListener(new MouseAdapter(){
			public void	mousePressed(MouseEvent e) {
				nodeFillColor();
			}
		});

	 	_rFont.setName("_rFont");
	 	_rFont.setOpaque(true);
	 	_rFont.setBackground(Color.white);
		_rFont.setBounds(111, 21, 109, 20);

	 	_rName.setName("_rName");
		_rName.setBackground(Color.white);
		_rName.setBounds(111, 41, 109, 20);
		_rName.setModel(fontsModel);
		_rName.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				fontNameChanged();
			}
		});

	 	_rSize.setName("_rSize");
	 	_rSize.setModel(sizeModel);
	 	_rSize.setSelectedIndex(3);
	 	_rSize.setBackground(Color.white);
		_rSize.setBounds(111, 61, 109, 20);
		_rSize.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				fontSizeChanged();
			}
		});

		_rBold.setName("_rBold");
	 	_rBold.setBackground(Color.white);
	 	_rBold.setBounds(111, 81, 109, 20);
	 	_rBold.setModel(new DefaultComboBoxModel<String>(new String[]{"true", "false"}));
	 	_rBold.setSelectedIndex(1);
		_rBold.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				fontStyleChanged();
			}
		});

	 	_rItalic.setName("_rItalic");
	 	_rItalic.setBackground(Color.white);
		_rItalic.setBounds(111, 101, 109, 20);
		_rItalic.setModel(new DefaultComboBoxModel<String>(new String[]{"true", "false"}));
		_rItalic.setSelectedIndex(1);
		_rItalic.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				fontStyleChanged();
			}
		});

	 	_rText.setName("_rText");
		_rText.setBackground(Color.white);
		_rText.setBounds(111, 121, 109, 20);
		_rText.addKeyListener(new KeyAdapter(){
			public void keyReleased(KeyEvent e){
				textBox = _rText.getText();
				for (DiagramNode node : diagram.getSelection().getNodes())
				{
					if (!(node instanceof ShapeNode))
						continue;
					
					ShapeNode shape = (ShapeNode)node;
					shape.setText(textBox);
				}

			}
		});

		_nodePropertyGrid.add(_lFillColor);
		_nodePropertyGrid.add(_lFont);
		_nodePropertyGrid.add(_lName);
		_nodePropertyGrid.add(_lSize);
		_nodePropertyGrid.add(_lBold);
		_nodePropertyGrid.add(_lItalic);
		_nodePropertyGrid.add(_lText);

		_nodePropertyGrid.add(_rFillColor);
		_nodePropertyGrid.add(_rFont);
		_nodePropertyGrid.add(_rName);
		_nodePropertyGrid.add(_rSize);
		_nodePropertyGrid.add(_rBold);
		_nodePropertyGrid.add(_rItalic);
		_nodePropertyGrid.add(_rText);

	}

	public void setNodeFillColor(Color color)
	{
		_rFillColor.setBackground(color);
	}

	public Color getNodeFillColor()
	{
		return _rFillColor.getBackground();
	}

	public void nodeFillColor()
	{
		ColorChooser _newColor = new ColorChooser(FlowCharterFrame.this);

		_newColor.setVisible(true);

		if(_newColor.getAccepted() == true)
		{
			setNodeFillColor(_newColor._colorChooser.getColor());
			for(DiagramNode node : diagram.getSelection().getNodes())
			{
				if (!(node instanceof ShapeNode))
					continue;
				
				ShapeNode shape = (ShapeNode)node;
				shape.setBrush(new SolidBrush(_newColor._colorChooser.getColor()));
			}
		}
	}

	Font boxFont;

	void fontSizeChanged()
	{
		int fontSize = (_rSize.getSelectedIndex() + 2) * 2;

		for (DiagramNode node : diagram.getSelection().getNodes())
		{
			if (!(node instanceof ShapeNode))
				continue;

			ShapeNodeStyle style = node.getStyle() != null ?
				(ShapeNodeStyle)node.getStyle() : new ShapeNodeStyle();
			if (fontSize != node.getEffectiveFont().getSize())
			{
				style.setFontSize(fontSize);
				node.setStyle(style);
			}
		}
	}

	void fontStyleChanged()
	{
		boolean bold = _rBold.getSelectedIndex() == 0;
		boolean italic = _rItalic.getSelectedIndex() == 0;

		int fontStyle = 0;
		if (bold)
			fontStyle |= Font.BOLD;
		if (italic)
			fontStyle |= Font.ITALIC;

		for (DiagramNode node : diagram.getSelection().getNodes())
		{
			if (!(node instanceof ShapeNode))
				continue;
			
			ShapeNodeStyle style = node.getStyle() != null ?
				(ShapeNodeStyle)node.getStyle() : new ShapeNodeStyle();
			if (fontStyle != node.getEffectiveFont().getStyle())
			{
				style.setFontStyle(fontStyle);
				node.setStyle(style);
			}
		}
	}

	void fontNameChanged()
	{
		String fontName = _fonts[_rName.getSelectedIndex()];

		for (DiagramNode node : diagram.getSelection().getNodes())
		{
			if (!(node instanceof ShapeNode))
				continue;
			
			ShapeNodeStyle style = node.getStyle() != null ?
				(ShapeNodeStyle)node.getStyle() : new ShapeNodeStyle();
			if (!fontName.equals(node.getEffectiveFont().getName()))
			{
				style.setFontFamily(fontName);
				node.setStyle(style);
			}
		}
	}

	public void createLinkPropertyGrid()
	{
		_lAHeadColor = new JLabel();
		_lAHeadSize = new JLabel();
		_lAEndMovable = new JLabel();
		_lAStartMovable = new JLabel();
		_lASegments = new JLabel();

		_rAHeadColor = new JPanel();
		_rAHeadSize = new JComboBox<String>();
		_rAEndMovable = new JComboBox<String>();
		_rAStartMovable = new JComboBox<String>();
		_rASegments = new JComboBox<String>();

		JLabel whiteSpace = new JLabel();

		//
		//create sizeModel
		//
		String[] size = {"4", "6", "8", "10", "12", "14", "16", "18"};
		DefaultComboBoxModel<String> sizeModel = new DefaultComboBoxModel<String>();

		for(String m : size){
			sizeModel.addElement(m);
		}

		String[] segments = {"1", "2", "3", "4", "5", "6", "7"};
		DefaultComboBoxModel<String> segmentModel = new DefaultComboBoxModel<String>();

		for(String m : segments){
			segmentModel.addElement(m);
		}

		//
		//ArrowPanel's elements
		//
		_lAHeadColor.setName("_lAHeadColor");
		_lAHeadColor.setText(" Head Color");
		_lAHeadColor.setBounds( 1, 1, 110, 20);
		_lAHeadColor.setOpaque(true);
		_lAHeadColor.setBorder(new LineBorder(new Color(100, 100, 100)));
		_lAHeadColor.setBackground(Color.white);


		_lAHeadSize.setName("_lAHeadSize");
		_lAHeadSize.setText(" Head size");
		_lAHeadSize.setBounds( 1, 21, 110, 20);
		_lAHeadSize.setOpaque(true);
		_lAHeadSize.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_lAHeadSize.setBackground(Color.white);

		_lAEndMovable.setName("_lAEndsMovable");
		_lAEndMovable.setText("Movable end");
		_lAEndMovable.setBounds(1, 41, 110, 20);
		_lAEndMovable.setOpaque(true);
		_lAEndMovable.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_lAEndMovable.setBackground(Color.white);

		_lAStartMovable.setName("_lAStartMovable");
		_lAStartMovable.setText("Movable start");
		_lAStartMovable.setBounds(1, 61, 110, 20);
		_lAStartMovable.setOpaque(true);
		_lAStartMovable.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_lAStartMovable.setBackground(Color.white);

		_lASegments.setName("_lASegments");
		_lASegments.setText(" Segments");
		_lASegments.setBounds( 1, 81, 110, 20);
		_lASegments.setOpaque(true);
		_lASegments.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_lASegments.setBackground(Color.white);

		_rAHeadColor.setName("_rAHeadColor");
		_rAHeadColor.setBounds( 111, 1, 110, 20);
		_rAHeadColor.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_rAHeadColor.setBackground(Color.white);
		_rAHeadColor.addMouseListener(new MouseAdapter(){
			public void mousePressed(MouseEvent e){
				arrowheadColor();
			}
		});

		_rAHeadSize.setName("_rAHeadSize");
		_rAHeadSize.setBounds( 111, 21, 110, 20);
		_rAHeadSize.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_rAHeadSize.setBackground(Color.white);
		_rAHeadSize.setModel(sizeModel);
		_rAHeadSize.setSelectedIndex(3);
		_rAHeadSize.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				arrowheadSizeChanged();
			}
		});

		_rAEndMovable.setName("__rAEndMovable");
		_rAEndMovable.setBounds( 111, 41, 110, 20);
		_rAEndMovable.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_rAEndMovable.setBackground(Color.white);
		_rAEndMovable.setModel(new DefaultComboBoxModel<String>(new String[]{"true", "false"}));
		_rAEndMovable.setSelectedIndex(0);
		_rAEndMovable.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				movableEndChanged();
			}
		});

		_rAStartMovable.setName("_rAStartMovable");
		_rAStartMovable.setBounds( 111, 61, 110, 20);
		_rAStartMovable.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_rAStartMovable.setBackground(Color.white);
		_rAStartMovable.setModel(new DefaultComboBoxModel<String>(new String[]{"true", "false"}));
		_rAStartMovable.setSelectedIndex(0);
		_rAStartMovable.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				movableStartChanged();
			}
		});

		_rASegments.setName("_rASegments");
		_rASegments.setBounds( 111, 81, 110, 20);
		_rASegments.setBorder(new LineBorder(new Color( 100, 100, 100)));
		_rASegments.setBackground(Color.white);
		_rASegments.setModel(segmentModel);
		_rASegments.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				segmentCountChanged();
			}
		});

		whiteSpace.setName("whiteSpace");
		whiteSpace.setBounds( 1, 101, 220, 40);
		whiteSpace.setOpaque(true);
		whiteSpace.setBackground(Color.white);

		_linkPropertyGrid.add(_lAHeadColor);
		_linkPropertyGrid.add(_lAHeadSize);
		_linkPropertyGrid.add(_lAEndMovable);
		_linkPropertyGrid.add(_lAStartMovable);
		_linkPropertyGrid.add(_lASegments);
		_linkPropertyGrid.add(_rAHeadColor);
		_linkPropertyGrid.add(_rAHeadSize);
		_linkPropertyGrid.add(_rAEndMovable);
		_linkPropertyGrid.add(_rAStartMovable);
		_linkPropertyGrid.add(_rASegments);
		_linkPropertyGrid.add(whiteSpace);

				// Overview
		overview = new Overview();
		overview.setDiagramView(diagramView);
		overview.setFitAll(true);
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();		
		
		// overviewFrame's frame
		overviewFrame = new JFrame();
		overviewFrame.setTitle("Overview");
		overviewFrame.setSize(230, 230);
		overviewFrame.setAlwaysOnTop(true);
		overviewFrame.setLayout(new java.awt.GridLayout(1,1));
		overviewFrame.add(overview);

		Point location = new Point(
			(int)screenSize.getWidth() - overviewFrame.getWidth() - 30,
			(int)screenSize.getHeight() - overviewFrame.getHeight() - 30);
		
		overviewFrame.setLocation(location);
	}

	public void arrowheadColor()
	{
		ColorChooser _newColor = new ColorChooser(FlowCharterFrame.this);

		_newColor.setVisible(true);

		
		if(_newColor.getAccepted() == true)
		{
			_rAHeadColor.setBackground(_newColor._colorChooser.getColor());
			for(DiagramLink link : diagram.getSelection().getLinks())
			{
				link.setBrush(new SolidBrush(_newColor._colorChooser.getColor()));
			}
		}
	}

	public void arrowheadSizeChanged()
	{
		float[] sizeFloat = { 4, 6, 8, 10, 12, 14, 16, 18};

		for (DiagramLink link : diagram.getSelection().getLinks())
			link.setHeadShapeSize(sizeFloat[_rAHeadSize.getSelectedIndex()]);
	}

	public void movableEndChanged()
	{
		if(_rAEndMovable.getSelectedIndex() == 1)
		{
			for(DiagramLink link: diagram.getSelection().getLinks())
				link.setAllowMoveEnd(false);
		}
		else
		{
			for(DiagramLink link : diagram.getSelection().getLinks())
				link.setAllowMoveEnd(true);
		}
	}

	public void movableStartChanged()
	{
		if(_rAStartMovable.getSelectedIndex() == 1)
		{
			for(DiagramLink link : diagram.getSelection().getLinks())
				link.setAllowMoveStart(false);
		}
		else
		{
			for(DiagramLink link : diagram.getSelection().getLinks())
				link.setAllowMoveStart(true);
		}
	}

	public void segmentCountChanged()
	{
		int[] segments = { 1, 2, 3, 4, 5, 6, 7};
		for(DiagramLink link : diagram.getSelection().getLinks()){
			if(link.getShape() == LinkShape.Cascading &&
										_rASegments.getSelectedIndex() == 0){
				_rASegments.setSelectedIndex(link.getSegmentCount() - 1);
				JOptionPane.showMessageDialog(this,"\"1\" cannot be selected ");

				return;
			}
			link.setSegmentCount(segments[_rASegments.getSelectedIndex()]);
		}
	}

	public void onNodeCreating(MouseEvent e)
	{
		Point nodeDevPos = new Point(e.getX(), e.getY());
		Point2D.Float nodeDocPos = diagramView.deviceToDoc(nodeDevPos);

		ShapeNode node = diagram.getFactory().createShapeNode(
			((int)nodeDocPos.getX() - 10), ((int)nodeDocPos.getY() - 10), 20, 20);

		AnchorPattern ap = FlowCharterFrame.getAnchorPattern();

		for(AnchorPoint point : ap.getPoints()){
			point.setMarkStyle(FlowCharterFrame.getAnchorStyle());
			point.setColor(_anchorColor);
		}

		node.setShape(FlowCharterFrame.getNodeShape());
		node.setAnchorPattern(ap);
		node.setHandlesStyle(HandlesStyle.DashFrame);

		FlowChartTransferHandler.setBoxCreate(false);
	}
	
	public JFrame getOverviewFrame()
	{
		return overviewFrame;
	}
	
	public Diagram diagram;
	private DiagramView diagramView;
	public Diagram hiddenDiagram;
	private Overview overview;
	private JFrame overviewFrame;	

    private JScrollPane _scrollConnectorsList;
    private JScrollPane _scrollNodeList;
	
	private static NodeInfo[] _nodes = null;
	private static ConnectorInfo[] _connectors = null;

	private static Color _anchorColor = Color.red;
	private static MarkStyle _anchorStyle = MarkStyle.Circle;

	private JMenuBar _menuBar;
	private JMenu _menuFile;
	private JMenuItem _menuItemNew;
	private JMenuItem _menuItemOpen;
	private JMenuItem _menuItemSave;
	private JMenuItem _menuItemPrint;
	private JMenuItem _menuItemPrintPreview;
	private JMenuItem _menuItemExit;
	private JMenu _menuEdit;
	private JMenuItem _menuItemUndo;
	private JMenuItem _menuItemRedo;
	private JMenuItem _menuHelp;
	private JMenuItem _menuItemAbout;
	private JMenuItem _menuTools;
	private JMenuItem _menuItemOptions;
	private Container _cp;
	private JScrollPane _scrollPane;
	private JPanel _rightPanel;
	private JPanel _leftPanel;
	private JPanel _tabRightPanel;
	private JPanel _nodePropertyGrid;
	private JPanel _linkPropertyGrid;

	private JTabbedPane _tabPane;
	private JList<String> _nodeList;
	private JComboBox<String> _connectorTypeCombo;
	private JList<String> _connectorList;
	private JFileChooser _fileChooser;
	private JPopupMenu _popupMenu;
	private JMenuItem _menuItemDelete;
	private FlowChartTransferHandler _transferHandler;

    Color defAnch = Color.red;

    boolean _editUndoRedo = false;
    
    public ListRenderer nodeRenderer;
    public ListRenderer connectorRenderer;

    private static int selectedNodeIndex;

 	String[] _typeArrow = { "Polyline", "Bezier", "Cascading"};
	
	private JLabel _lFillColor;
	private JLabel _lFont;
	private JLabel _lName;
	private JLabel _lSize;
	private JLabel _lBold;
	private JLabel _lItalic;
	private JLabel _lText;
	
	private JPanel _rFillColor;
	private JLabel _rFont;
	private JComboBox<String>  _rName;
	private JComboBox<String> _rSize;
	private JComboBox<String> _rBold;
	private JComboBox<String> _rItalic;
	private JTextField _rText;
	
	String textBox;
	
	private JLabel _lAHeadColor;
	private JLabel _lAHeadSize;
	private JLabel _lAEndMovable;
	private JLabel _lAStartMovable;
	private JLabel _lASegments;
	
	private JPanel _rAHeadColor;
	private JComboBox<String> _rAHeadSize;
	private JComboBox<String> _rAEndMovable;
	private JComboBox<String> _rAStartMovable;
	private JComboBox<String> _rASegments;
	
	private float arrowHeadSize = 10;
	
	private static final long serialVersionUID = 1L;
}